@@ -1,7 +1,7 @@
 import typer
 from rich import print
 from typing import Annotated, Optional
-from meshagent.cli.common_options import ProjectIdOption, ApiKeyIdOption, RoomOption
+from meshagent.cli.common_options import ProjectIdOption, RoomOption
 import json
 import aiohttp
 from meshagent.api import (
@@ -9,16 +9,20 @@ from meshagent.api import (
     ParticipantToken,
     WebSocketClientProtocol,
     ParticipantGrant,
+    ApiScope,
 )
 from meshagent.api.helpers import meshagent_base_url, websocket_room_url
 from meshagent.api.services import send_webhook
 from meshagent.cli import async_typer
-from meshagent.cli.helper import get_client, resolve_project_id
-from meshagent.cli.helper import resolve_api_key, resolve_room
+from meshagent.cli.helper import get_client, resolve_project_id, resolve_key
+from meshagent.cli.helper import resolve_room
 from urllib.parse import urlparse
 from pathlib import PurePath
 import socket
 import ipaddress
+import pathlib
+from pydantic_yaml import parse_yaml_raw_as
+from meshagent.api.participant_token import ParticipantTokenSpec
 
 app = async_typer.AsyncTyper()
 
@@ -72,7 +76,6 @@ async def make_call(
     *,
     project_id: ProjectIdOption = None,
     room: RoomOption,
-    api_key_id: ApiKeyIdOption = None,
     role: str = "agent",
     local: Optional[bool] = None,
     agent_name: Annotated[
@@ -87,12 +90,76 @@ async def make_call(
     arguments: Annotated[
         str, typer.Option(..., help="JSON string with arguments for the call")
     ] = {},
+    permissions: Annotated[
+        Optional[str],
+        typer.Option(
+            "--permissions",
+            "-p",
+            help="File path to a token definition, if not specified default agent permissions will be used",
+        ),
+    ] = None,
+    key: Annotated[
+        str,
+        typer.Option("--key", help="an api key to sign the token with"),
+    ] = None,
+):
+    key = await resolve_key(project_id=project_id, key=key)
+
+    if permissions is not None:
+        with open(str(pathlib.Path(permissions).expanduser().resolve()), "rb") as f:
+            spec = parse_yaml_raw_as(ParticipantTokenSpec, f.read())
+
+            token = ParticipantToken(
+                name=spec.identity,
+            )
+            token.add_role_grant(role=role)
+            token.add_room_grant(room)
+            token.add_api_grant(spec.api)
+
+    else:
+        token = None
+
+    await _make_call(
+        project_id=project_id,
+        room=room,
+        role=role,
+        local=local,
+        agent_name=agent_name,
+        name=name,
+        participant_name=participant_name,
+        url=url,
+        arguments=arguments,
+        token=token,
+        key=key,
+    )
+
+
+async def _make_call(
+    *,
+    project_id: ProjectIdOption = None,
+    room: RoomOption,
+    role: str = "agent",
+    local: Optional[bool] = None,
+    agent_name: Annotated[
+        Optional[str], typer.Option(..., help="deprecated and unused", hidden=True)
+    ] = None,
+    name: Annotated[str, typer.Option(..., help="deprecated", hidden=True)] = None,
+    participant_name: Annotated[
+        Optional[str],
+        typer.Option(..., help="the participant name to be used by the callee"),
+    ] = None,
+    url: Annotated[str, typer.Option(..., help="URL the agent should call")],
+    arguments: Annotated[
+        str, typer.Option(..., help="JSON string with arguments for the call")
+    ] = {},
+    token: Optional[ParticipantToken] = None,
+    permissions: Optional[ApiScope] = None,
+    key: str,
 ):
     """
     Instruct an agent to 'call' a given URL with specific arguments.
 
     """
-
     if name is not None:
         print("[yellow]name is deprecated and should no longer be passed[/yellow]")
 
@@ -109,21 +176,17 @@ async def make_call(
     account_client = await get_client()
     try:
         project_id = await resolve_project_id(project_id=project_id)
-        api_key_id = await resolve_api_key(project_id, api_key_id)
+
         room = resolve_room(room)
 
-        key = (
-            await account_client.decrypt_project_api_key(
-                project_id=project_id, id=api_key_id
+        if token is None:
+            token = ParticipantToken(
+                name=participant_name,
             )
-        )["token"]
-
-        token = ParticipantToken(
-            name=participant_name, project_id=project_id, api_key_id=api_key_id
-        )
-        token.add_role_grant(role=role)
-        token.add_room_grant(room)
-        token.grants.append(ParticipantGrant(name="tunnel_ports", scope="9000"))
+            token.add_api_grant(permissions or ApiScope.agent_default())
+            token.add_role_grant(role=role)
+            token.add_room_grant(room)
+            token.grants.append(ParticipantGrant(name="tunnel_ports", scope="9000"))
 
         if local is None:
             local = is_local_url(url)
@@ -134,7 +197,7 @@ async def make_call(
                 data = {
                     "room_url": websocket_room_url(room_name=room),
                     "room_name": room,
-                    "token": token.to_jwt(token=key),
+                    "token": token.to_jwt(api_key=key),
                     "arguments": arguments,
                 }
 
@@ -148,7 +211,7 @@ async def make_call(
                     url=websocket_room_url(
                         room_name=room, base_url=meshagent_base_url()
                     ),
-                    token=token.to_jwt(token=key),
+                    token=token.to_jwt(api_key=key),
                 )
             ) as client:
                 print("[bold green]Making agent call...[/bold green]")
