@@ -1,36 +1,42 @@
 import typer
 from rich import print
 from typing import Annotated, Optional
+from meshagent.tools import Toolkit
+from meshagent.tools.storage import StorageToolkitBuilder
 from meshagent.cli.common_options import (
     ProjectIdOption,
-    ApiKeyIdOption,
     RoomOption,
 )
-from meshagent.tools import Toolkit
-from meshagent.api import RoomClient, WebSocketClientProtocol
+from meshagent.api import (
+    RoomClient,
+    WebSocketClientProtocol,
+    ParticipantToken,
+    ApiScope,
+)
 from meshagent.api.helpers import meshagent_base_url, websocket_room_url
 from meshagent.cli import async_typer
 from meshagent.cli.helper import (
     get_client,
     resolve_project_id,
-    resolve_api_key,
-    resolve_token_jwt,
     resolve_room,
+    resolve_key,
 )
-from meshagent.agents.chat import ChatBot
+
 from meshagent.openai import OpenAIResponsesAdapter
-from meshagent.api.services import ServiceHost
-from meshagent.computers.agent import ComputerAgent
-from meshagent.agents.chat import (
-    ChatBotThreadOpenAIImageGenerationTool,
-    ChatBotThreadLocalShellTool,
-)
 
 from typing import List
 from pathlib import Path
 
-from meshagent.openai.tools.responses_adapter import WebSearchTool
+from meshagent.openai.tools.responses_adapter import (
+    WebSearchToolkitBuilder,
+    MCPToolkitBuilder,
+    WebSearchTool,
+    LocalShellConfig,
+    WebSearchConfig,
+)
+
 from meshagent.api import RequiredToolkit, RequiredSchema
+from meshagent.api.services import ServiceHost
 
 app = async_typer.AsyncTyper(help="Join a chatbot to a room")
 
@@ -43,11 +49,31 @@ def build_chatbot(
     toolkit: List[str],
     schema: List[str],
     image_generation: Optional[str] = None,
-    local_shell: bool,
-    computer_use: bool,
-    web_search: bool,
+    local_shell: Optional[str] = None,
+    computer_use: Optional[str] = None,
+    web_search: Optional[str] = None,
+    mcp: Optional[str] = None,
+    storage: Optional[str] = None,
+    require_image_generation: Optional[str] = None,
+    require_local_shell: Optional[str] = None,
+    require_computer_use: Optional[str] = None,
+    require_web_search: Optional[str] = None,
+    require_mcp: Optional[str] = None,
+    require_storage: Optional[str] = None,
     rules_file: Optional[str] = None,
 ):
+    from meshagent.agents.chat import ChatBot
+
+    from meshagent.tools.storage import StorageToolkit
+
+    from meshagent.agents.chat import (
+        ChatBotThreadOpenAIImageGenerationToolkitBuilder,
+        ChatBotThreadLocalShellToolkitBuilder,
+        ChatBotThreadOpenAIImageGenerationTool,
+        ChatBotThreadLocalShellTool,
+        ImageGenerationConfig,
+    )
+
     requirements = []
 
     toolkits = []
@@ -67,8 +93,13 @@ def build_chatbot(
 
     BaseClass = ChatBot
     if computer_use:
-        BaseClass = ComputerAgent
+        from meshagent.computers.agent import ComputerAgent
 
+        if ComputerAgent is None:
+            raise RuntimeError(
+                "Computer use is enabled, but meshagent.computers is not installed."
+            )
+        BaseClass = ComputerAgent
         llm_adapter = OpenAIResponsesAdapter(
             model=model,
             response_options={
@@ -77,7 +108,9 @@ def build_chatbot(
             },
         )
     else:
-        llm_adapter = OpenAIResponsesAdapter(model=model)
+        llm_adapter = OpenAIResponsesAdapter(
+            model=model,
+        )
 
     class CustomChatbot(BaseClass):
         def __init__(self):
@@ -90,32 +123,77 @@ def build_chatbot(
             )
 
         async def get_thread_toolkits(self, *, thread_context, participant):
-            toolkits = await super().get_thread_toolkits(
-                thread_context=thread_context, participant=participant
-            )
+            providers = []
 
-            thread_toolkit = Toolkit(name="thread_toolkit", tools=[])
-
-            if local_shell:
-                thread_toolkit.tools.append(
-                    ChatBotThreadLocalShellTool(thread_context=thread_context)
-                )
-
-            if image_generation is not None:
-                print("adding openai image gen to thread", flush=True)
-                thread_toolkit.tools.append(
+            if require_image_generation:
+                providers.append(
                     ChatBotThreadOpenAIImageGenerationTool(
-                        model=image_generation,
                         thread_context=thread_context,
-                        partial_images=3,
+                        config=ImageGenerationConfig(
+                            name="image_generation",
+                            partial_images=3,
+                        ),
                     )
                 )
 
-            if web_search:
-                thread_toolkit.tools.append(WebSearchTool())
+            if require_local_shell:
+                providers.append(
+                    ChatBotThreadLocalShellTool(
+                        thread_context=thread_context,
+                        config=LocalShellConfig(name="local_shell"),
+                    )
+                )
 
-            toolkits.append(thread_toolkit)
-            return toolkits
+            if require_mcp:
+                raise Exception(
+                    "mcp tool cannot be required by cli currently, use 'optional' instead"
+                )
+
+            if require_web_search:
+                providers.append(
+                    WebSearchTool(config=WebSearchConfig(name="web_search"))
+                )
+
+            if require_storage:
+                providers.extend(StorageToolkit().tools)
+
+            tk = await super().get_thread_toolkits(
+                thread_context=thread_context, participant=participant
+            )
+            return [
+                *(
+                    [Toolkit(name="tools", tools=providers)]
+                    if len(providers) > 0
+                    else []
+                ),
+                *tk,
+            ]
+
+        async def get_thread_toolkit_builders(self, *, thread_context, participant):
+            providers = []
+
+            if image_generation:
+                providers.append(
+                    ChatBotThreadOpenAIImageGenerationToolkitBuilder(
+                        thread_context=thread_context
+                    )
+                )
+
+            if local_shell:
+                providers.append(
+                    ChatBotThreadLocalShellToolkitBuilder(thread_context=thread_context)
+                )
+
+            if mcp:
+                providers.append(MCPToolkitBuilder())
+
+            if web_search:
+                providers.append(WebSearchToolkitBuilder())
+
+            if storage:
+                providers.append(StorageToolkitBuilder())
+
+            return providers
 
     return CustomChatbot
 
@@ -124,11 +202,9 @@ def build_chatbot(
 async def make_call(
     *,
     project_id: ProjectIdOption = None,
-    room: RoomOption = None,
-    api_key_id: ApiKeyIdOption = None,
+    room: RoomOption,
     role: str = "agent",
     agent_name: Annotated[str, typer.Option(..., help="Name of the agent to call")],
-    token_path: Annotated[Optional[str], typer.Option()] = None,
     rule: Annotated[List[str], typer.Option("--rule", "-r", help="a system rule")] = [],
     rules_file: Optional[str] = None,
     toolkit: Annotated[
@@ -141,7 +217,7 @@ async def make_call(
     ] = [],
     model: Annotated[
         str, typer.Option(..., help="Name of the LLM model to use for the chatbot")
-    ] = "gpt-4o",
+    ] = "gpt-5",
     image_generation: Annotated[
         Optional[str], typer.Option(..., help="Name of an image gen model")
     ] = None,
@@ -157,21 +233,59 @@ async def make_call(
     web_search: Annotated[
         Optional[bool], typer.Option(..., help="Enable web search tool calling")
     ] = False,
+    mcp: Annotated[
+        Optional[bool], typer.Option(..., help="Enable mcp tool calling")
+    ] = False,
+    storage: Annotated[
+        Optional[bool], typer.Option(..., help="Enable storage toolkit")
+    ] = False,
+    require_image_generation: Annotated[
+        Optional[str], typer.Option(..., help="Name of an image gen model", hidden=True)
+    ] = None,
+    require_computer_use: Annotated[
+        Optional[bool],
+        typer.Option(
+            ...,
+            help="Enable computer use (requires computer-use-preview model)",
+            hidden=True,
+        ),
+    ] = False,
+    require_local_shell: Annotated[
+        Optional[bool],
+        typer.Option(..., help="Enable local shell tool calling", hidden=True),
+    ] = False,
+    require_web_search: Annotated[
+        Optional[bool],
+        typer.Option(..., help="Enable web search tool calling", hidden=True),
+    ] = False,
+    require_mcp: Annotated[
+        Optional[bool], typer.Option(..., help="Enable mcp tool calling", hidden=True)
+    ] = False,
+    require_storage: Annotated[
+        Optional[bool], typer.Option(..., help="Enable storage toolkit", hidden=True)
+    ] = False,
+    key: Annotated[
+        str,
+        typer.Option("--key", help="an api key to sign the token with"),
+    ] = None,
 ):
+    key = await resolve_key(project_id=project_id, key=key)
     account_client = await get_client()
     try:
         project_id = await resolve_project_id(project_id=project_id)
-        api_key_id = await resolve_api_key(project_id, api_key_id)
         room = resolve_room(room)
-        jwt = await resolve_token_jwt(
-            project_id=project_id,
-            api_key_id=api_key_id,
-            token_path=token_path,
+
+        token = ParticipantToken(
             name=agent_name,
-            role=role,
-            room=room,
         )
 
+        token.add_api_grant(ApiScope.agent_default())
+
+        token.add_role_grant(role=role)
+        token.add_room_grant(room)
+
+        jwt = token.to_jwt(api_key=key)
+
         print("[bold green]Connecting to room...[/bold green]", flush=True)
         async with RoomClient(
             protocol=WebSocketClientProtocol(
@@ -195,9 +309,16 @@ async def make_call(
                 rule=rule,
                 toolkit=toolkit,
                 schema=schema,
+                rules_file=rules_file,
                 image_generation=image_generation,
                 web_search=web_search,
-                rules_file=rules_file,
+                mcp=mcp,
+                storage=storage,
+                require_web_search=require_web_search,
+                require_local_shell=require_local_shell,
+                require_image_generation=require_image_generation,
+                require_mcp=require_mcp,
+                require_storage=require_storage,
             )
 
             bot = CustomChatbot()
@@ -219,7 +340,6 @@ async def make_call(
 @app.async_command("service")
 async def service(
     *,
-    room: RoomOption = None,
     agent_name: Annotated[str, typer.Option(..., help="Name of the agent to call")],
     rule: Annotated[List[str], typer.Option("--rule", "-r", help="a system rule")] = [],
     rules_file: Optional[str] = None,
@@ -233,7 +353,7 @@ async def service(
     ] = [],
     model: Annotated[
         str, typer.Option(..., help="Name of the LLM model to use for the chatbot")
-    ] = "gpt-4o",
+    ] = "gpt-5",
     image_generation: Annotated[
         Optional[str], typer.Option(..., help="Name of an image gen model")
     ] = None,
@@ -249,12 +369,41 @@ async def service(
     web_search: Annotated[
         Optional[bool], typer.Option(..., help="Enable web search tool calling")
     ] = False,
+    mcp: Annotated[
+        Optional[bool], typer.Option(..., help="Enable mcp tool calling")
+    ] = False,
+    storage: Annotated[
+        Optional[bool], typer.Option(..., help="Enable storage toolkit")
+    ] = False,
+    require_image_generation: Annotated[
+        Optional[str], typer.Option(..., help="Name of an image gen model", hidden=True)
+    ] = None,
+    require_computer_use: Annotated[
+        Optional[bool],
+        typer.Option(
+            ...,
+            help="Enable computer use (requires computer-use-preview model)",
+            hidden=True,
+        ),
+    ] = False,
+    require_local_shell: Annotated[
+        Optional[bool],
+        typer.Option(..., help="Enable local shell tool calling", hidden=True),
+    ] = False,
+    require_web_search: Annotated[
+        Optional[bool],
+        typer.Option(..., help="Enable web search tool calling", hidden=True),
+    ] = False,
+    require_mcp: Annotated[
+        Optional[bool], typer.Option(..., help="Enable mcp tool calling", hidden=True)
+    ] = False,
+    require_storage: Annotated[
+        Optional[bool], typer.Option(..., help="Enable storage toolkit", hidden=True)
+    ] = False,
     host: Annotated[Optional[str], typer.Option()] = None,
     port: Annotated[Optional[int], typer.Option()] = None,
     path: Annotated[str, typer.Option()] = "/agent",
 ):
-    room = resolve_room(room)
-
     print("[bold green]Connecting to room...[/bold green]", flush=True)
 
     service = ServiceHost(host=host, port=port)
@@ -268,9 +417,16 @@ async def service(
             rule=rule,
             toolkit=toolkit,
             schema=schema,
+            rules_file=rules_file,
             web_search=web_search,
             image_generation=image_generation,
-            rules_file=rules_file,
+            mcp=mcp,
+            storage=storage,
+            require_web_search=require_web_search,
+            require_local_shell=require_local_shell,
+            require_image_generation=require_image_generation,
+            require_mcp=require_mcp,
+            require_storage=require_storage,
         ),
     )
 
