@@ -1,6 +1,5 @@
 import typer
 import asyncio
-from typing import Optional
 
 from meshagent.cli import async_typer
 
@@ -22,10 +21,15 @@ from meshagent.cli import cli_mcp
 from meshagent.cli import chatbot
 from meshagent.cli import voicebot
 from meshagent.cli import mailbot
+from meshagent.cli import containers
+from meshagent.cli import oauth2
+from meshagent.cli import helpers
+from meshagent.cli import meeting_transcriber
 from meshagent.cli.exec import register as register_exec
 from meshagent.cli.version import __version__
+from meshagent.cli.helper import get_active_api_key
+from meshagent.otel import otel_config
 
-from meshagent.cli import otel
 
 from art import tprint
 
@@ -34,16 +38,15 @@ import logging
 import os
 import sys
 from pathlib import Path
-from meshagent.cli.helper import get_client, resolve_project_id, resolve_api_key
 
+otel_config(service_name="meshagent-cli")
 
-otel.init(level=logging.INFO)
 
 # Turn down OpenAI logs, they are a bit noisy
 logging.getLogger("openai").setLevel(logging.ERROR)
 logging.getLogger("httpx").setLevel(logging.ERROR)
 
-app = async_typer.AsyncTyper()
+app = async_typer.AsyncTyper(no_args_is_help=True)
 app.add_typer(call.app, name="call")
 app.add_typer(auth.app, name="auth")
 app.add_typer(projects.app, name="project")
@@ -62,6 +65,10 @@ app.add_typer(cli_mcp.app, name="mcp")
 app.add_typer(chatbot.app, name="chatbot")
 app.add_typer(voicebot.app, name="voicebot")
 app.add_typer(mailbot.app, name="mailbot")
+app.add_typer(containers.app, name="container")
+app.add_typer(oauth2.app, name="oauth2")
+app.add_typer(helpers.app, name="helpers")
+app.add_typer(meeting_transcriber.app, name="meeting-transcriber")
 
 register_exec(app)
 
@@ -142,66 +149,6 @@ def version():
     print(__version__)
 
 
-@app.command(
-    "env",
-    help="Generate shell commands to set meshagent environment variables.",
-)
-def env(
-    shell: Optional[str] = typer.Option(
-        None,
-        "--shell",
-        case_sensitive=False,
-        help="bash | zsh | fish | powershell | cmd",
-    ),
-    unset: bool = typer.Option(
-        False, "--unset", help="Output commands to unset the variables."
-    ),
-):
-    """Print shell-specific exports/unsets for Docker environment variables."""
-
-    async def command():
-        nonlocal shell, unset
-        shell = (shell or detect_shell()).lower()
-        if shell not in SHELL_RENDERERS:
-            typer.echo(f"Unsupported shell '{shell}'.", err=True)
-            raise typer.Exit(code=1)
-
-        client = await get_client()
-        try:
-            project_id = await resolve_project_id(project_id=None)
-            api_key_id = await resolve_api_key(project_id=project_id, api_key_id=None)
-
-            token = (
-                await client.decrypt_project_api_key(
-                    project_id=project_id, id=api_key_id
-                )
-            )["token"]
-        finally:
-            await client.close()
-
-        vars = {
-            "MESHAGENT_PROJECT_ID": project_id,
-            "MESHAGENT_KEY_ID": api_key_id,
-            "MESHAGENT_SECRET": token,
-        }
-        if shell not in SHELL_RENDERERS:
-            typer.echo(f"Unsupported shell '{shell}'.", err=True)
-            raise typer.Exit(code=1)
-
-        render = SHELL_RENDERERS[shell]
-
-        for name, value in vars.items():
-            typer.echo(render(name, value, unset))
-
-        if not unset and shell in ("bash", "zsh"):
-            typer.echo(
-                "\n# Run this command to configure your current shell:\n"
-                '# eval "$(meshagent env)"'
-            )
-
-    _run_async(command())
-
-
 @app.command("setup")
 def setup_command():
     """Perform initial login and project/api key activation."""
@@ -215,11 +162,20 @@ def setup_command():
         project_id = await projects.activate(None, interactive=True)
         if project_id is None:
             print("You have choosen to not activate a project. Exiting.")
-        if project_id is not None:
-            print("Activate an api-key...")
-            api_key_id = await api_keys.activate(None, interactive=True)
-            if api_key_id is None:
-                print("You have choosen to not activate an api-key. Exiting.")
+        if (
+            project_id is not None
+            and await get_active_api_key(project_id=project_id) is None
+        ):
+            if typer.confirm(
+                "You do not have an active api key for this project. Would you like to create and activate a new api key?",
+                default=True,
+            ):
+                name = typer.prompt(
+                    "Enter a name for your API Key (must be a unique name):"
+                )
+                await api_keys.create(
+                    project_id=None, activate=True, silent=True, name=name
+                )
 
     _run_async(runner())
 
