@@ -14,18 +14,10 @@ ProjectIdOption = Annotated[
     ),
 ]
 
-ApiKeyIdOption = Annotated[
-    Optional[str],
-    typer.Option(
-        "--api-key-id",
-        help="A MeshAgent project API key id. If empty, the activated api key will be used.",
-    ),
-]
-
 RoomOption = Annotated[
-    Optional[str],
+    str,
     typer.Option(
         "--room",
-        help="Room name. If empty, the MESHAGENT_ROOM environment variable will be used.",
+        help="Room name",
     ),
 ]
